﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using Goldammer;
using System.IO;
using System.Windows.Forms.DataVisualization.Charting;
using Goldammer.MAIExtensions.DAQ;
using System.Globalization;
using System.Data.SqlServerCe;
using System.Data.Common;
using NI_DDC_NET20_DLL;
using System.Data.SqlClient;

namespace DAQExtensionTest {

    public partial class FormDataSinkTest : Form {
        public FormDataSinkTest() {
            InitializeComponent();
        }
        MAIDevice dev = MAI.Devices[0];
        bool m_bRunning = false;
        private void timer1_Tick(object sender, EventArgs e) {
            if (!m_bRunning)
                return;
            if (dev == null)
                return;
            label6.Text = dev.ADChannels.DataSink.BurstCount.ToString();
        }


        private void StartMeasure() {
            if (dev == null)
                return;
            buttonConvert.Enabled = false;
            buttonReadRAW.Enabled = false;
            dev.StartMeasure();
            m_bRunning = true;
            button1.Text = "Stop";
        }

        private void StopMeasure() {
            buttonConvert.Enabled = true;
            buttonReadRAW.Enabled = true;
            if (dev == null)
                return;
            m_bRunning = false;
            dev.StopMeasure();
            dev.ADChannels.DataSink.Disable();
            dev.CTChannels.DataSink.Disable();

            CloseGolbalFilesAndConnectoins();
            dev.ADChannels.DataSink.Target = null;
            dev.CTChannels.DataSink.Target = null;
            dev.ClearAllChannelLists();
            button1.Text = "Start";

        }

        private void ConfigureMeasurementChannels() {
            dev.ClearAllChannelLists();
            int numberOfADChannels = 0;
            Int32.TryParse(textBoxAD.Text, out numberOfADChannels);
            for (int i = 0; i < numberOfADChannels; i++) {
                dev.ADChannels[i].CreateMeasurementChannel(OversamplingMode.Disabled, GainFactor.Disabled, true);

            }
            int numberOfcTChannels = 0;
            Int32.TryParse(textBoxCT.Text, out numberOfcTChannels);
            for (int i = 0; i < numberOfcTChannels; i++) {
                dev.CTChannels[i].CreateImpulseCounter(010,ImpulseDirection.Up,SignalEdge.Rising,false);

            }

            int numberOfTTLChannels = 0;
            Int32.TryParse(textBoxTTL.Text, out numberOfTTLChannels);
            for (int i = 0; i < numberOfTTLChannels; i++) {
                dev.TTLChannels[i].CreateMeasurementChannel(MeasurementChannelSyncMode.OnADChannels,TTLDigitalType.Bit);

            }
            dev.CTChannels.SetSampleRate(100);
            dev.ADChannels.SetSampleRate(100);
        }

        private void CloseGolbalFilesAndConnectoins() {
            if (myconn != null) {

                myconn.Close();
                myconn.Dispose();
                myconn = null;
            }
            if (myTDMS != null) {
                myTDMS.Write();

                myTDMS.Close();
                myTDMS.Dispose();
                myTDMS = null;
            }
        }   

        private void FormDataSinkTest_Load(object sender, EventArgs e) {
            if (dev == null)
                return;
                
            labelDev.Text = dev.Info.SerialNumber;
            dev.ClearAllChannelLists();

        }


        #region Explicit Definition of a TDMSTarget
        NI_DDCFile myTDMS = null;
        private IDataSinkTarget CreateExplicitTDM() {
            DateTime dt1 = DateTime.Now;
            MAIDataSinkChannelMapTDMS NIChannelMap =  new MAIDataSinkChannelMapTDMS();
            try {
                string Filename = "nationalMessdaten.tdm";
                if (File.Exists(Filename))
                    File.Delete(Filename);
                myTDMS = NI_DDC.CreateNIDDCFile(Filename, Path.GetExtension(Filename) == ".tdms", "NameTestMessung", "TestMessDecs", "TitelMess", "Frank");
                try {
                    NI_DDCChannelGroup adChannels = myTDMS.AddChannelGroup("AD Kanäle", "Skaliert");
                    adChannels.SetNIWaveFormProperty("None");
                    for (int i = 0; i < dev.ADChannels.MeasurementChannelsCount; i++) {
                        MAIMeasurementChannelAD cad = dev.ADChannels.GetMeasurementChannel((uint)i);
                        NI_DDCChannel c = adChannels.AddChannel(NI_DDCDatatype.Double, "AD"+i.ToString(), cad.Name, cad.Unit);
                        c.SetNIProperties("ADKanal"+i.ToString(), 1, dt1, dt1, "t");
                        c.SetNIWaveFormProperties(1.0 / dev.ADChannels.GetSampleRate(), 0, 0.0, dt1, "relative", cad.Name, cad.Unit);
                        NIChannelMap.AddTargetForChannel(cad, myTDMS, c, null);
                    }

                    myTDMS.AddProperty("Starttime", NI_DDCDatatype.Timestamp, dt1);
                    NIChannelMap.SetTargetParameterForStartTime(myTDMS, "Starttime");

          
                }
                finally {
                    myTDMS.Write();
                  
 
                }
            }
            catch (Exception ex) {
                MessageBox.Show(ex.Message);
            }

            return MAIDataSinkTarget.CreateTargetTDMS(NIChannelMap);
        }

        #endregion

        #region explicit definition od an SQL Target
        DbConnection myconn = null;

        private IDataSinkTarget CreateExplicitSQLCe() {
            if (File.Exists("MyFile.MAIdata")) {
                File.Delete("MyFile.MAIdata");
            }
            SqlCeEngine en = new SqlCeEngine("datasource=MyFile.MAIdata");
            en.CreateDatabase();
            en.Dispose();
            myconn = new SqlCeConnection("datasource=MyFile.MAIdata");
            myconn.Open();
            DbCommand nuCommand = myconn.CreateCommand();
            string createCmdPar = "";
            string createCmdParamNames = "";
            string createCmdColumNames = "";
            for (int i = 0; i < dev.ADChannels.MeasurementChannelsCount; i++) {
                createCmdPar += ", ADScaled" + i.ToString() + " Real ";
                createCmdColumNames += ", ADScaled" + i.ToString();
                createCmdParamNames += ", @ADS" + i.ToString();
            }
            createCmdColumNames = createCmdColumNames.Substring(2);
            createCmdParamNames = createCmdParamNames.Substring(2);
            nuCommand.CommandText = "CREATE TABLE measureData  (ID BIGINT identity(1,1) PRIMARY KEY "+createCmdPar+");";
            nuCommand.ExecuteNonQuery();
            nuCommand.CommandText = "CREATE TABLE metaDataMaster  (ID INT identity(1,1) PRIMARY KEY, Start DateTime, Samplerate Real, NumberOfBursts BIGINT);";
            nuCommand.ExecuteNonQuery();
            //nuCommand.CommandText = "CREATE TABLE metaDataDetail  (ID INT identity(1,1) PRIMARY KEY, RefID INT, Name nvarchar(256), Value Real, Desc nvarchar(256));";
            //nuCommand.ExecuteNonQuery();
            

            MAIDataSinkChannelMapSQL MyChannelmap = new MAIDataSinkChannelMapSQL();
            DbCommand InsertDataCommand = myconn.CreateCommand();
            InsertDataCommand.CommandText = "Insert into measureData (" + createCmdColumNames + ") values (" + createCmdParamNames + ");";
              
            for (int i = 0; i < dev.ADChannels.MeasurementChannelsCount; i++) {
               DbParameter param = InsertDataCommand.CreateParameter();
               param.DbType = DbType.Double;
               param.ParameterName = "@ADS" + i.ToString();
               InsertDataCommand.Parameters.Add(param);
               MyChannelmap.AddDBStatementForData(dev.ADChannels.GetMeasurementChannel((uint)i), InsertDataCommand, param, null);
            }
            DbCommand InsertMDCommand = myconn.CreateCommand();
            InsertMDCommand.CommandText = "Insert into metaDataMaster (Start, Samplerate) values ( @StTime, @SamRat);";
            DbParameter paramStrat = InsertDataCommand.CreateParameter();
            paramStrat.DbType = DbType.DateTime;
            paramStrat.ParameterName = "@StTime";
            paramStrat.Value = DateTime.MinValue;
            InsertMDCommand.Parameters.Add(paramStrat);
            MyChannelmap.SetDBStatementForStartTime(InsertMDCommand, paramStrat);
            DbParameter paramSR = InsertDataCommand.CreateParameter();
            paramSR.DbType = DbType.Double;
            paramSR.ParameterName = "@SamRat";
            paramSR.Value = -1.0;

            InsertMDCommand.Parameters.Add(paramSR);
            MyChannelmap.SetDBStatementForSampleRate(InsertMDCommand, paramSR);
            DbCommand UpdateMDCommand = myconn.CreateCommand();
            UpdateMDCommand.CommandText = "update metaDataMaster set NumberOfBursts = @NumOBursts where ID = ALL (Select max(ID) from metaDataMaster);";
            DbParameter paramNB = UpdateMDCommand.CreateParameter();
            paramNB.DbType = DbType.Int64;
            paramNB.ParameterName = "@NumOBursts";
            paramNB.Value = 0;
            UpdateMDCommand.Parameters.Add(paramNB);
            MyChannelmap.SetDBStatementForNumberOfBurstsRecorded(UpdateMDCommand, paramNB);
            return MAIDataSinkTarget.CreateTargetSQL(MyChannelmap);
        }

        #endregion



        private void buttonConvert_Click(object sender, EventArgs e) {
            if (m_bRunning) {
                return;
            }
       
           MAIDataSinkTarget.ConvertRAW(  MAIDataSinkTarget.CreateTargetCSV("KonvertierteMessdaten.csv", "KonvertierteKanalinfos.csv", false, ';', new CultureInfo("en-US"), false),"Rohdatenmessung");

        }

        private void buttonReadRAW_Click(object sender, EventArgs e) {
            string Info = "";
            try {
                MAIRAWDataReader dr = new MAIRAWDataReader("Rohdatenmessung");
                Info = "ADChannels: " + dr.ADChannelCount.ToString() + Environment.NewLine;
                Info += "Bursts    : " + dr.BurstCount.ToString() + Environment.NewLine;

                double[] mittelwert = new double[dr.ADChannelCount];
                for (int i = 0; i < (int)dr.BurstCount; i++) {
                    for (int ii = 0; ii < dr.ADChannelCount; ii++)
                        mittelwert[ii] += dr.AD[ii].ReadDouble((ulong)i);
                }
                for (int ii = 0; ii < dr.ADChannelCount; ii++) {
                    mittelwert[ii] /= dr.BurstCount;
                    Info += "mittelwert[" + ii + "] : " + mittelwert[ii].ToString() + Environment.NewLine;
                }
                MessageBox.Show(Info);

            }
            catch (Exception Ex) {
                MessageBox.Show(Info + Environment.NewLine + Ex.Message);
            }
            
        }

        private void button1_Click(object sender, EventArgs e) {
            try {
                if (m_bRunning) {
                    StopMeasure();
                    return;
                }
                ConfigureMeasurementChannels();
                dev.ADChannels.DataSink.Target = CreateTargetByRadioButton();
                dev.ConfigMeasure();
                dev.ADChannels.DataSink.Enable(DateTime.Now);

                StartMeasure();
            }
            catch (Exception ex) {
                MessageBox.Show(ex.Message);
            }
        }

        private IDataSinkTarget CreateTargetByRadioButton() {
            if (rbTargetCSV.Checked)
                return CreateTargetCSV();
            if (rbTargetRAW.Checked)
                return CreateTargetRAW();
            if (rbTargetSQL.Checked)
                return CreateTargetSQL();
            if (rbTargetSQLServer.Checked)
                return CreateTargetSQLServer();
            if (rbTargetSQLCe.Checked)
                return CreateTargetSQLCe();
            if (rbTargetTDMS.Checked)
                return CreateTargetTDMS();
            if (rbTargetTAFFMAT.Checked)
                return CreateTargetTaffMat();
            if (rbTargetWAV.Checked)
                return CreateTargetWAV();
            return null;
        }


   

 
        private IDataSinkTarget CreateTargetCSV() {
            
            // Modify Parameters and see results.
            return MAIDataSinkTarget.CreateTargetCSV("Messdaten.csv", "Kanalinfos.csv", false, ';', new CultureInfo("en-US"), false);

        }       
        
        private IDataSinkTarget CreateTargetRAW() {
            #region Cleanup old Output
            string[] alte = Directory.GetFiles(".", "Rohdatenmessung.*");
            if (alte != null)
                foreach (string s in alte)
                    File.Delete(s);
            #endregion

            return MAIDataSinkTarget.CreateTargetRAW("Rohdatenmessung", 100000000);
        }

        private IDataSinkTarget CreateTargetSQL() {
            return CreateExplicitSQLCe();
        }

        private IDataSinkTarget CreateTargetSQLServer() {
            string standard = "Server=myServerAddress;Database=myDataBase;User Id=myUsername;Password=myPassword;";
            string trusted = "Server=myServerAddress;Database=myDataBase;Trusted_Connection=True;";
            string SpecifyServerInstance = @"Server=myServerName\myInstanceName;Database=myDataBase;User Id=myUsername;Password=myPassword;";

            // These are just example strings. You must provide valid connection infos to a database you can access

            SqlConnection conn = new SqlConnection(standard);
            return MAIDataSinkTarget.CreateTargetSQLServer(conn,"MAIN_Table", "Data", "ChannelInfos", true);
        }

        private IDataSinkTarget CreateTargetSQLCe() {
             string s = "Output.sdf";
             #region delete old Output

             if(File.Exists(s))
                File.Delete(s);
             #endregion

            return MAIDataSinkTarget.CreateTargetSQLCeFile(s);
        }

        private IDataSinkTarget CreateTargetTDMS() {
            string[] alte = Directory.GetFiles(".", "*.TDM");
            if (alte != null)
                foreach (string s in alte)
                    File.Delete(s);
            alte = Directory.GetFiles(".", "*.TDMS");
            if (alte != null)
                foreach (string s in alte)
                    File.Delete(s);
           return MAIDataSinkTarget.CreateTargetTDMS("TDMS_File", "MeasurementName", "Desctription", "Title", "Author");
            // or maybe create an Explicit Mapping :
           //    return CreateExplicitTDM();
        }

        private IDataSinkTarget CreateTargetTaffMat() {
            #region Cleanup old Output
            string[] alte = Directory.GetFiles(".", "*.DAT");
            if (alte != null)
                foreach (string s in alte)
                    File.Delete(s);
            if (File.Exists("TaffmatExample.hdr"))
                File.Delete("TaffmatExample.hdr");
            #endregion

            return MAIDataSinkTarget.CreateTargetTAFFMat("TaffmatExample");
            
        }
        private IDataSinkTarget CreateTargetWAV() {
            #region Cleanup old Output
            string[] alte = Directory.GetFiles(".", "*.WAV");
            if (alte != null)
                foreach (string s in alte)
                    File.Delete(s);
            #endregion

            return MAIDataSinkTarget.CreateTargetWAV("AudioRecording", 100000000);
        }

    }
}
