﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using Goldammer.MAIExtensions.DAQ;
using Goldammer.Helper.OPC;
using Goldammer;
using System.Globalization;

namespace TEST_MAI_NET35 {
    public partial class FormOPCClients : Form {
        public FormOPCClients() {
            InitializeComponent();
        }

        private void button1_Click(object sender, EventArgs e) {
            if (m_bRunning) {
                StopMeasure();
                button1.Text = "Start";
            }
            else {
                StartMeasure();
                button1.Text = "Stop";

            }
            rbDA.Enabled = !m_bRunning;
            rbUA.Enabled = !m_bRunning;
            rbXi.Enabled = !m_bRunning;

        }

        private void FormOPCClients_Load(object sender, EventArgs e) {

        }

        private void ChangeTarget(object sender, EventArgs e) {
            tbDA_URL.Enabled = rbDA.Checked;
            tbUA_URL.Enabled = rbUA.Checked;
            tbXiURL.Enabled  = rbXi.Checked;
            checkBoxTCP.Enabled = rbXi.Checked;
            tbXiInstance.Enabled = !checkBoxTCP.Checked;
        }

        private MAIDataSinkChannelMapOPCBase CreateSelectedMap(MAIChannelControllerAD Controller) {
            uint ServerUpdateRate = uint.Parse(tbServerUpdateRate.Text);   

            if (rbDA.Checked) {
                Opc.Da.Server serv = MAIOPCConnectionDA.BuildAndConnect(tbDA_URL.Text, new System.Net.NetworkCredential());
                MAIDataSinkChannelMapOPCNET20 ret = new MAIDataSinkChannelMapOPCNET20(serv, ServerUpdateRate);
                Dictionary<String, Type> channels = MAIOPCConnectionDA.GetWriteableTagNamesAndDataType(serv);
                for(int i= 0; i< Controller.MeasurementChannelsCount; i++){
                    MAIMeasurementChannelAD mchan = Controller.GetMeasurementChannel((uint)i);
                    string opcname = "ADChannels.Device1.Analog" + (i + 1).ToString();
                    foreach(KeyValuePair<String, Type> c in channels){
                        if (c.Key.Contains(opcname))
                            ret.AddTargetForChannel(mchan, c.Key, null);
                    }
                }
                return ret;
            }
            else if(rbUA.Checked){
                Opc.Ua.Client.Session sessi = MAIOPCConnectionUA.BuildSessionAndConnect(tbUA_URL.Text);
                MAIDataSinkChannelMapOPCUA ret = new MAIDataSinkChannelMapOPCUA(sessi, ServerUpdateRate);
                Dictionary<Opc.Ua.NodeId, Type> channels = MAIOPCConnectionUA.BrowseSessionForWriteableTags(sessi);
                for (int i = 0; i < Controller.MeasurementChannelsCount; i++) {
                    MAIMeasurementChannelAD mchan = Controller.GetMeasurementChannel((uint)i);
                    string opcname = "ADChannels.Device1.Analog" + (i + 1).ToString();
                    foreach (KeyValuePair<Opc.Ua.NodeId, Type> c in channels) {
                        if (c.Key.Identifier.ToString().Contains(opcname))
                            ret.AddTargetForChannel(mchan, c.Key, null);
                    }
                }
                return ret;
            }
            else if(rbXi.Checked){
                Xi.Client.Base.API.IXiContext conti = MAIOPCConnectionXi.BuildContextFromDiscoveryServerInfoAndSubscribeToWriteEndpoint(tbXiURL.Text, tbXiInstance.Text, MAIOPCConnectionXiType.TCP);
                MAIDataSinkChannelMapOPCXi ret = new MAIDataSinkChannelMapOPCXi(conti, ServerUpdateRate);
                Dictionary<Xi.Contracts.Data.InstanceId, Type> channels = MAIOPCConnectionXi.BrowseContextForWriteableTags(conti);
                for (int i = 0; i < Controller.MeasurementChannelsCount; i++) {
                    MAIMeasurementChannelAD mchan = Controller.GetMeasurementChannel((uint)i);
                    string opcname = "ADChannels.Device1.Analog" + (i + 1).ToString();
                    foreach (KeyValuePair<Xi.Contracts.Data.InstanceId, Type> c in channels) {
                        if (c.Key.FullyQualifiedId.Contains(opcname))
                            ret.AddTargetForChannel(mchan, c.Key, null);
                    }
                }
                return ret;
            }

            throw new Exception("CreateSelectedMap: unexpected error.");
        
        }


        private void StartMeasure() {
            if (dev == null)
                dev = MAI.Devices[0];
            dev.ClearAllChannelLists();

            dev.ADChannels[0].CreateMeasurementChannel(OversamplingMode.Disabled, GainFactor.Disabled, true);
            dev.ADChannels[1].CreateMeasurementChannel(OversamplingMode.Disabled, GainFactor.Disabled, true);
            dev.ADChannels[2].CreateMeasurementChannel(OversamplingMode.Disabled, GainFactor.Disabled, true);
            dev.ADChannels[3].CreateMeasurementChannel(OversamplingMode.Disabled, GainFactor.Disabled, true);

            dev.ADChannels.SetSampleRate(100);
            MAIDataSinkChannelMapOPCBase myMAP = CreateSelectedMap(dev.ADChannels);

             dev.ADChannels.DataSink.Target = MAIDataSinkTarget.CreateTargetOPC(myMAP);
            dev.ConfigMeasure();
            dev.ADChannels.DataSink.Enable(DateTime.Now);
            dev.CTChannels.DataSink.Enable(DateTime.Now);

            dev.StartMeasure();
            m_bRunning = true;
        }

        private void StopMeasure() {
            if (dev == null)
                return;
            m_bRunning = false;
            dev.StopMeasure();
            dev.ADChannels.DataSink.Disable();
            dev.CTChannels.DataSink.Disable();

            dev.ADChannels.DataSink.Target = null;
            dev.CTChannels.DataSink.Target = null;
            dev.ClearAllChannelLists();
        }

        MAIDevice dev = null;
        bool m_bRunning = false;

        private void timer1_Tick(object sender, EventArgs e) {
            if (!m_bRunning)
                return;
            if (dev == null)
                return;
            if (dev.ADChannels.DataSink.Target == null)
                return;

            tbBurstsrecorded.Text = dev.ADChannels.DataSink.BurstCount.ToString();
            
            

        }

        private void checkBoxTCP_CheckedChanged(object sender, EventArgs e) {
            tbXiInstance.Enabled = !checkBoxTCP.Checked;
        }
     
    }
}
