﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Windows.Forms.DataVisualization.Charting;
using Goldammer;
using Goldammer.MAIExtensions.DAQ;
using System.Globalization;

namespace DatasinkPreviewDemo {
    public partial class MAIN : Form {
        MAIDevice m_dev;
        
        public MAIN() {
            InitializeComponent();
            m_dev = MAI.Devices[0];
            if (m_dev != null) {
                foreach (MAIChannelAD s in m_dev.ADChannels)
                    checkedListBox1.Items.Add(s.HardwareChannelNumber, true);
            }
        }



        int NumberOfSeries = 0;
        double[] RMSArray;
        double[] MAXArray;
        private void SetupChart() {
            NumberOfSeries = checkedListBox1.CheckedItems.Count;
            RMSArray = new double[NumberOfSeries];
            MAXArray = new double[NumberOfSeries];
            chart1.Series.Clear();
            chart1.ChartAreas.Clear();
            ChartArea na;
            Series ns;
            for (int i = 0; i < NumberOfSeries; i++) { 
                ns = chart1.Series.Add(i.ToString());
                ns.ChartType = SeriesChartType.FastLine;

                // Der folgende Teil konfiguriert das Chart zum RogaREC lookalike:
                // wenn Sie das auskommentieren, wird alles in ein Koodinatensystem gemalt
                ns.Color = Color.Red;
                na = chart1.ChartAreas.Add(i.ToString());
                ns.ChartArea = na.Name;
                na.AlignmentOrientation = AreaAlignmentOrientations.Horizontal;
                na.Position.X = 5;
                na.Position.Y = (100 / checkedListBox1.CheckedItems.Count) * i;
                na.Position.Width = 90;
                na.Position.Height = 100 / checkedListBox1.CheckedItems.Count;
                na.AxisX.LabelStyle.Enabled = false;
                na.AxisX.MajorTickMark.Enabled = false;
                na.AxisX.Enabled = AxisEnabled.False;
                na.AxisY.Maximum = 10;
                na.AxisY.Minimum = -10;
                na.AxisY.MajorGrid.Interval = 10;
                na.AxisY.MinorGrid.Interval = 1;
                na.AxisY.MinorGrid.Enabled = true;
                na.AxisY.MinorGrid.LineColor = Color.LightGray;
                na.AxisY.MinorTickMark.Enabled = true;
                na.AxisY.MajorTickMark.Interval = 10;
                na.AxisY.LabelStyle.Interval = 10;

            }

        }

        int DisplayPoints;
        int DisplayRate;

        private void button3_Click(object sender, EventArgs e) {
            // ändert die Konfiguration bzw. startet die Anzeige beim ersten Klick
            timer1.Enabled = false;
            StopMeasurement();
            SetupChart();
            DisplayPoints = Convert.ToInt32(textBoxDisplayPoints.Text);
            DisplayRate = Convert.ToInt32(textBoxDisplayRate.Text);
            StartMeasurement();
            timer1.Enabled = true;
        }

        private void StopMeasurement() {
            if (m_dev == null)
                return;
            m_dev.ADChannels.DataSink.Disable();
            m_dev.StopMeasure();
            m_dev.ClearAllChannelLists();
        }

        private void StartMeasurement() {
            if (m_dev == null)
                return;
            m_dev.ADChannels.DataSink.Disable();
            m_dev.StopMeasure(); // Messung anhalten
            m_dev.ClearAllChannelLists();  // Kanallisten löschen
            m_dev.ADChannels.DataSink.Target = MAIDataSinkTarget.CreateTargetCSV(textBoxOutFile.Text, false, ' ', new CultureInfo("en-US")); // CSV Target anhängen
            for (int i = 0; i < checkedListBox1.CheckedIndices.Count; i++) { // Messkanäle definieren
                m_dev.ADChannels[checkedListBox1.CheckedIndices[i]].CreateMeasurementChannel(OversamplingMode.Disabled, GainFactor.Disabled, true);
            }
            m_dev.ADChannels.SetSampleRate(Convert.ToDouble(textBoxSamplerate.Text)); // Sample Rate setzen
            m_dev.ConfigMeasure();
            m_dev.ADChannels.DataSink.Enable(DateTime.Now);  // Datasink starten
            // jetzt das Preview konfigurieren:
            m_dev.ADChannels.DataSink.Preview.MaxTickCount  = (uint)DisplayPoints;
            m_dev.ADChannels.DataSink.Preview.TickFrequency = DisplayRate;
            m_dev.ADChannels.DataSink.Preview.Enabled = true;
            // und nur Daten aufzeichnen, wenn "Recording" im UI aktiviert ist:
            m_dev.ADChannels.DataSink.OutputEnabled = checkBox1.Checked;
            m_dev.StartMeasure();
        }

        private void timer1_Tick(object sender, EventArgs e) {
            try {
                int i = 0;
                m_dev.ADChannels.DataSink.Preview.Update();    // Preview aktualisieren
                foreach (Series s in chart1.Series) {  // für alle Graphen
                    RMSArray[i] = m_dev.ADChannels.DataSink.Preview.GetRootMeanSquare(i);
                    MAXArray[i] = m_dev.ADChannels.DataSink.Preview.GetBufferMax(i);
                    s.Points.DataBindXY(m_dev.ADChannels.DataSink.Preview.Timestamps, m_dev.ADChannels.DataSink.Preview[i]); // Setze die Punkte aus dem Datasink Preview
                    i++;
                }
                chart2.Series["RMS"].Points.DataBindY(RMSArray);
                chart2.Series["Maximum"].Points.DataBindY(MAXArray);

            }
            catch {
            }
        }

        private void MAIN_FormClosed(object sender, FormClosedEventArgs e) {
            timer1.Enabled = false;
            StopMeasurement();
        }

        private void checkBox1_CheckedChanged(object sender, EventArgs e) {
            if (m_dev == null)
                return;
            m_dev.ADChannels.DataSink.OutputEnabled = checkBox1.Checked;  // Wenn "Record" hineingedrückt ist, werden Daten geschrieben, sonst nicht. 
        }

        private void rbVIewChart_CheckedChanged(object sender, EventArgs e) {
            chart1.Visible = rbVIewChart.Checked;
            chart2.Visible = rbViewBars.Checked;
        }
       
    


      
    }
}
