﻿using System;
using System.Collections.Generic;
using System.Text;
using OpcCom.Da;
using System.Net;


namespace Goldammer.Helper.OPC {
    /// <summary>
    /// This class is compatible to .net Framework 2.0
    /// </summary>
    public class MAIOPCConnectionDA {

        #region private

        private static void BrowseForChildTags(Opc.Da.Server ServerToBrowse, string ItemName, Dictionary<string, Type> Dict) {

            Opc.Da.BrowseFilters filt = new Opc.Da.BrowseFilters();
            filt.MaxElementsReturned = 1000;
            filt.ReturnAllProperties = true;
            filt.ReturnPropertyValues = true;
            filt.ElementNameFilter = "";
            filt.BrowseFilter = Opc.Da.browseFilter.all;
            Opc.Da.Item id = new Opc.Da.Item();
            id.ItemName = ItemName;
            Opc.Da.BrowsePosition pos;
            Opc.Da.BrowseElement[] elms = ServerToBrowse.Browse(id, filt, out  pos);
            if (elms != null) {
                foreach (Opc.Da.BrowseElement e in elms) {
                    if (e.HasChildren) {
                        BrowseForChildTags(ServerToBrowse, e.ItemName, Dict);
                    }
                    if (e.Properties != null) {
                        if (!IsWriteable(e))
                            continue;
                        if (e.Properties[0].Description.Contains("Item Canonical Data Type")) {
                            System.Object s = e.Properties[0].Value;
                            Dict.Add(e.ItemName, (System.Type)s);
                        }
                    }

                }
            }

        }

        private static bool IsWriteable(Opc.Da.BrowseElement Elem) {
            foreach (Opc.Da.ItemProperty p in Elem.Properties) {
                if (p.Description.Contains("Item Access Rights")) {
                    if ((((int)p.Value) & 2) == 2)
                        return true;
                }
            }
            return false;
        }

        #endregion

        public static Opc.Da.Server BuildAndConnect(String ServerURL, NetworkCredential Credentials) {
            Opc.URL url            = new Opc.URL(ServerURL);
	        OpcCom.Factory factory = new OpcCom.Factory(true);
	        Opc.Da.Server server   = new Opc.Da.Server(factory,url);
	        server.Connect(url, new Opc.ConnectData(Credentials));
            return server;
        }

        public static  Dictionary<String,Type>  GetWriteableTagNamesAndDataType(Opc.Da.Server ServerToBrowse){
	        Dictionary<String,Type>  ret = new Dictionary<String,Type> ();
            BrowseForChildTags(ServerToBrowse, "", ret);
	        return ret;
        }

    }
}
