﻿using System;
using System.Collections.Generic;
using System.Text;
using Xi.Client.Base.API;
using Xi.Client.Base;
using System.ServiceModel.Description;
using Xi.Contracts.Constants;
using Xi.Contracts.Data;
using System.ServiceModel;

namespace Goldammer.Helper.OPC {

    public enum MAIOPCConnectionXiType {
        TCP,
        NamedPipes
    };


    /// <summary>
    ///  This class is compatible to .net Framework 3.0 (System.ServiceModel)
    /// </summary>
    public class MAIOPCConnectionXi {

        #region private

        #region fields

        private static string m_prefixTCP  = "net.tcp://";
        private static string m_prefixPipe = "net.pipe://";

        #endregion

        private static void AppendWriteableLeafs(IXiContext SessionToBrowse, FindCriteria FindCriteria, Dictionary<InstanceId, Type> Dict, bool SkipUnderscoreNames, uint NumberToReturn) {
            List<ObjectAttributes> objAttrs = new List<ObjectAttributes>();
            List<ObjectAttributes> returnedObjs = null;

            bool GetStartingObjectAttrs = false;
            IEnumerable<ObjectAttributes> enumOA = SessionToBrowse.FindObjects(FindCriteria, NumberToReturn);
            if (enumOA != null) {
                returnedObjs = new List<ObjectAttributes>(enumOA);
                while (returnedObjs != null) {
                    bool firstInList = true;
                    foreach (ObjectAttributes obj in returnedObjs) {
                        if ((FindCriteria.FilterSet != null) && (GetStartingObjectAttrs) && (firstInList)) {
                            //skip the first in the list if it contains
                            // the starting object attributes.
                            firstInList = false;
                        }
                        else {
                            if ((SkipUnderscoreNames) && (obj.Name.StartsWith("_")))
                                continue;
                            objAttrs.Add(obj);
                        }
                    }
                    if (returnedObjs.Count == NumberToReturn) {
                        enumOA = SessionToBrowse.FindObjects(null, NumberToReturn);
                        if (enumOA != null)
                            returnedObjs = new List<ObjectAttributes>(enumOA);
                        else
                            returnedObjs = null;
                    }
                    else
                        break; // break if no more to get
                }

                foreach (ObjectAttributes oa in objAttrs) {
                    if (oa.IsLeaf) {
                        if (!oa.NotAccessible) {// Is accessible
                            if (oa.IsWritable) {
                                string datatype = oa.DataTypeId.LocalId;
                                Type t = Type.GetType(datatype);
                                Dict.Add(oa.InstanceId, t);
                            }
                        }
                    }
                    else {// a branch
                        if (!oa.NotAccessible) {
                            FindCriteria FindCriteria2 = new FindCriteria();
                            FindCriteria2.StartingPath = new ObjectPath(oa.InstanceId.FullyQualifiedId, null);
                            AppendWriteableLeafs(SessionToBrowse, FindCriteria2, Dict, SkipUnderscoreNames, NumberToReturn);
                        }
                    }
                }
            }
        }

        #endregion

        public static IXiContext BuildContextFromDiscoveryServerInfoAndSubscribeToWriteEndpoint(string DiscoveryServerURL, string OPCServerName, MAIOPCConnectionXiType ConnectionType) {
            return BuildContextFromDiscoveryServerInfoAndSubscribeToWriteEndpoint(DiscoveryServerURL, OPCServerName, ConnectionType,ContextOptions.NoOptions, 100000, (uint)System.Threading.Thread.CurrentThread.CurrentCulture.LCID, Guid.NewGuid().ToString(), 10, new TimeSpan(0, 0, 0, 01));
        }

        public static IXiContext BuildContextFromDiscoveryServerInfoAndSubscribeToWriteEndpoint(string DiscoveryServerURL, string OPCServerName, MAIOPCConnectionXiType ConnectionType, ContextOptions Options, uint ContextTimeOut, uint LocaleID, string ReInitiateKey, uint WriteEndpointKeepaliveSkipCount, TimeSpan WriteEndpointCallbackRate) {
            string serverPrefix = "";
            if (ConnectionType == MAIOPCConnectionXiType.NamedPipes)
                serverPrefix = m_prefixPipe;
            else
                serverPrefix = m_prefixTCP;
          

            XiEndpointDiscovery EndpointDiscovery = new XiEndpointDiscovery(DiscoveryServerURL);
            IEnumerable<ServiceEndpoint> seps = EndpointDiscovery.ResourceManagementServiceEndpoints;
            ServiceEndpoint sep = null;

            foreach (ServiceEndpoint serviEnd in seps) {
                if (ConnectionType == MAIOPCConnectionXiType.NamedPipes) {
                    string acturi = serviEnd.Address.Uri.ToString().ToLower();
                    if (acturi.Contains(serverPrefix) && (acturi.Contains(OPCServerName.ToLower())))
                        sep = serviEnd;
                }
                else {
                    sep = serviEnd;
                }
            }
            if ((sep == null)) {
                throw new Exception("Endpoint with name " + OPCServerName + " not found in server configuration.");

            }
            return BuildContextFromDiscoveryServerInfoAndSubscribeToWriteEndpoint(EndpointDiscovery, sep, Options, ContextTimeOut, LocaleID, ReInitiateKey, WriteEndpointKeepaliveSkipCount,WriteEndpointCallbackRate);
        }

        public static IXiContext BuildContextFromDiscoveryServerInfoAndSubscribeToWriteEndpoint(XiEndpointDiscovery EndpointDiscovery, ServiceEndpoint ResourceEndpoint, ContextOptions Options, uint ContextTimeOut, uint LocaleID, string ReInitiateKey, uint WriteEndpointKeepaliveSkipCount, TimeSpan WriteEndpointCallbackRate) {
            string serverPrefix = "";          
            if (ResourceEndpoint.Address.Uri.AbsoluteUri.StartsWith(m_prefixPipe))
                serverPrefix = m_prefixPipe;
            if (ResourceEndpoint.Address.Uri.AbsoluteUri.StartsWith(m_prefixTCP))
                serverPrefix = m_prefixTCP;
            if (String.IsNullOrEmpty(serverPrefix))
                throw new Exception("Unknown Protocol.");
            IXiContext Context = XiContext.Initiate(ResourceEndpoint, EndpointDiscovery.ServerEntry, ContextTimeOut, (uint)Options, LocaleID, ReInitiateKey);
            string[] tmp = ResourceEndpoint.Address.Uri.AbsolutePath.Split(new char[]{'/'},StringSplitOptions.RemoveEmptyEntries);
            string ServerName = tmp[0].ToLower();
            if (Context != null) {
                EndpointDiscovery.RankReadWriteSubscribeEndpoints(ResourceEndpoint);
                List<ServiceEndpoint> wserv = new List<ServiceEndpoint>(EndpointDiscovery.WriteServiceEndpoints);
                foreach (ServiceEndpoint serviEnd in wserv) {
                    string acturi = serviEnd.Address.Uri.ToString().ToLower();
                    if (acturi.Contains(serverPrefix) && (acturi.Contains(ServerName))) {
                        IXiEndpointBase xiEndpoint = null;
                        try {
                            xiEndpoint = Context.OpenEndpoint(serviEnd,WriteEndpointKeepaliveSkipCount,  WriteEndpointCallbackRate);
                            
                        }
                        catch (Exception ex) {
                            xiEndpoint = null;
                            throw new Exception("Failed to connect to write endpoint.",ex);
                        }

                    }
                }
            }
            return Context;
        }

        public static Dictionary<InstanceId, Type> BrowseContextForWriteableTags(IXiContext SessionToBrowse) {
            //if (SessionToBrowse.WriteEndpoint == null)
            //    throw new Exception("WriteEndpoint is not connected !");
            Dictionary<InstanceId, Type> ret = new Dictionary<InstanceId, Type>();
            uint NumberToReturn = 111111;
            try {
                // used to test starting at a specific DA InstanceId
                FindCriteria FindCriteria = new FindCriteria();
                FindCriteria.StartingPath = new ObjectPath("DA://", null);
                AppendWriteableLeafs(SessionToBrowse, FindCriteria, ret, true, NumberToReturn);



            }
            catch (FaultException<XiFault> fex) {
                if (fex.Detail.ErrorCode == XiFaultCodes.E_WRAPPEDSERVER_NOT_ACCESSIBLE) {
                    // TODO: Gray out or otherwise handle wrapped server shutdown

                }
            }
            catch (Exception ex) {

            }
            return ret;
        }

     
     }
}
