﻿Imports System.Text
Imports Xi.Client.Base.API
Imports Xi.Client.Base
Imports System.ServiceModel.Description
Imports Xi.Contracts.Constants
Imports Xi.Contracts.Data
Imports System.ServiceModel

Namespace Goldammer.Helper.OPC

	Public Enum MAIOPCConnectionXiType
		TCP
		NamedPipes
	End Enum


	''' <summary>
	'''  This class is compatible to .net Framework 3.0 (System.ServiceModel)
	''' </summary>
	Public Class MAIOPCConnectionXi

		#Region "private"

		#Region "fields"

		Private Shared m_prefixTCP As String = "net.tcp://"
		Private Shared m_prefixPipe As String = "net.pipe://"

		#End Region

		Private Shared Sub AppendWriteableLeafs(ByVal SessionToBrowse As IXiContext, ByVal FindCriteria As FindCriteria, ByVal Dict As Dictionary(Of InstanceId, Type), ByVal SkipUnderscoreNames As Boolean, ByVal NumberToReturn As UInteger)
			Dim objAttrs As New List(Of ObjectAttributes)()
			Dim returnedObjs As List(Of ObjectAttributes) = Nothing

			Dim GetStartingObjectAttrs As Boolean = False
			Dim enumOA As IEnumerable(Of ObjectAttributes) = SessionToBrowse.FindObjects(FindCriteria, NumberToReturn)
			If enumOA IsNot Nothing Then
				returnedObjs = New List(Of ObjectAttributes)(enumOA)
				Do While returnedObjs IsNot Nothing
					Dim firstInList As Boolean = True
					For Each obj As ObjectAttributes In returnedObjs
						If (FindCriteria.FilterSet IsNot Nothing) AndAlso (GetStartingObjectAttrs) AndAlso (firstInList) Then
							'skip the first in the list if it contains
							' the starting object attributes.
							firstInList = False
						Else
							If (SkipUnderscoreNames) AndAlso (obj.Name.StartsWith("_")) Then
								Continue For
							End If
							objAttrs.Add(obj)
						End If
					Next obj
					If returnedObjs.Count = NumberToReturn Then
						enumOA = SessionToBrowse.FindObjects(Nothing, NumberToReturn)
						If enumOA IsNot Nothing Then
							returnedObjs = New List(Of ObjectAttributes)(enumOA)
						Else
							returnedObjs = Nothing
						End If
					Else
						Exit Do ' break if no more to get
					End If
				Loop

				For Each oa As ObjectAttributes In objAttrs
					If oa.IsLeaf Then
						If Not oa.NotAccessible Then ' Is accessible
							If oa.IsWritable Then
								Dim datatype As String = oa.DataTypeId.LocalId
								Dim t As Type = Type.GetType(datatype)
								Dict.Add(oa.InstanceId, t)
							End If
						End If
					Else ' a branch
						If Not oa.NotAccessible Then
							Dim FindCriteria2 As New FindCriteria()
							FindCriteria2.StartingPath = New ObjectPath(oa.InstanceId.FullyQualifiedId, Nothing)
							AppendWriteableLeafs(SessionToBrowse, FindCriteria2, Dict, SkipUnderscoreNames, NumberToReturn)
						End If
					End If
				Next oa
			End If
		End Sub

		#End Region

		Public Shared Function BuildContextFromDiscoveryServerInfoAndSubscribeToWriteEndpoint(ByVal DiscoveryServerURL As String, ByVal OPCServerName As String, ByVal ConnectionType As MAIOPCConnectionXiType) As IXiContext
			Return BuildContextFromDiscoveryServerInfoAndSubscribeToWriteEndpoint(DiscoveryServerURL, OPCServerName, ConnectionType,ContextOptions.NoOptions, 100000, CUInt(System.Threading.Thread.CurrentThread.CurrentCulture.LCID), Guid.NewGuid().ToString(), 10, New TimeSpan(0, 0, 0, 01))
		End Function

		Public Shared Function BuildContextFromDiscoveryServerInfoAndSubscribeToWriteEndpoint(ByVal DiscoveryServerURL As String, ByVal OPCServerName As String, ByVal ConnectionType As MAIOPCConnectionXiType, ByVal Options As ContextOptions, ByVal ContextTimeOut As UInteger, ByVal LocaleID As UInteger, ByVal ReInitiateKey As String, ByVal WriteEndpointKeepaliveSkipCount As UInteger, ByVal WriteEndpointCallbackRate As TimeSpan) As IXiContext
			Dim serverPrefix As String = ""
			If ConnectionType = MAIOPCConnectionXiType.NamedPipes Then
				serverPrefix = m_prefixPipe
			Else
				serverPrefix = m_prefixTCP
			End If


			Dim EndpointDiscovery As New XiEndpointDiscovery(DiscoveryServerURL)
			Dim seps As IEnumerable(Of ServiceEndpoint) = EndpointDiscovery.ResourceManagementServiceEndpoints
			Dim sep As ServiceEndpoint = Nothing

			For Each serviEnd As ServiceEndpoint In seps
				If ConnectionType = MAIOPCConnectionXiType.NamedPipes Then
					Dim acturi As String = serviEnd.Address.Uri.ToString().ToLower()
					If acturi.Contains(serverPrefix) AndAlso (acturi.Contains(OPCServerName.ToLower())) Then
						sep = serviEnd
					End If
				Else
					sep = serviEnd
				End If
			Next serviEnd
			If (sep Is Nothing) Then
				Throw New Exception("Endpoint with name " & OPCServerName & " not found in server configuration.")

			End If
			Return BuildContextFromDiscoveryServerInfoAndSubscribeToWriteEndpoint(EndpointDiscovery, sep, Options, ContextTimeOut, LocaleID, ReInitiateKey, WriteEndpointKeepaliveSkipCount,WriteEndpointCallbackRate)
		End Function

		Public Shared Function BuildContextFromDiscoveryServerInfoAndSubscribeToWriteEndpoint(ByVal EndpointDiscovery As XiEndpointDiscovery, ByVal ResourceEndpoint As ServiceEndpoint, ByVal Options As ContextOptions, ByVal ContextTimeOut As UInteger, ByVal LocaleID As UInteger, ByVal ReInitiateKey As String, ByVal WriteEndpointKeepaliveSkipCount As UInteger, ByVal WriteEndpointCallbackRate As TimeSpan) As IXiContext
			Dim serverPrefix As String = ""
			If ResourceEndpoint.Address.Uri.AbsoluteUri.StartsWith(m_prefixPipe) Then
				serverPrefix = m_prefixPipe
			End If
			If ResourceEndpoint.Address.Uri.AbsoluteUri.StartsWith(m_prefixTCP) Then
				serverPrefix = m_prefixTCP
			End If
			If String.IsNullOrEmpty(serverPrefix) Then
				Throw New Exception("Unknown Protocol.")
			End If
			Dim Context As IXiContext = XiContext.Initiate(ResourceEndpoint, EndpointDiscovery.ServerEntry, ContextTimeOut, CUInt(Options), LocaleID, ReInitiateKey)
			Dim tmp() As String = ResourceEndpoint.Address.Uri.AbsolutePath.Split(New Char(){"/"c},StringSplitOptions.RemoveEmptyEntries)
			Dim ServerName As String = tmp(0).ToLower()
			If Context IsNot Nothing Then
				EndpointDiscovery.RankReadWriteSubscribeEndpoints(ResourceEndpoint)
				Dim wserv As New List(Of ServiceEndpoint)(EndpointDiscovery.WriteServiceEndpoints)
				For Each serviEnd As ServiceEndpoint In wserv
					Dim acturi As String = serviEnd.Address.Uri.ToString().ToLower()
					If acturi.Contains(serverPrefix) AndAlso (acturi.Contains(ServerName)) Then
						Dim xiEndpoint As IXiEndpointBase = Nothing
						Try
							xiEndpoint = Context.OpenEndpoint(serviEnd,WriteEndpointKeepaliveSkipCount, WriteEndpointCallbackRate)

						Catch ex As Exception
							xiEndpoint = Nothing
							Throw New Exception("Failed to connect to write endpoint.",ex)
						End Try

					End If
				Next serviEnd
			End If
			Return Context
		End Function

		Public Shared Function BrowseContextForWriteableTags(ByVal SessionToBrowse As IXiContext) As Dictionary(Of InstanceId, Type)
			'if (SessionToBrowse.WriteEndpoint == null)
			'    throw new Exception("WriteEndpoint is not connected !");
			Dim ret As New Dictionary(Of InstanceId, Type)()
			Dim NumberToReturn As UInteger = 111111
			Try
				' used to test starting at a specific DA InstanceId
				Dim FindCriteria As New FindCriteria()
				FindCriteria.StartingPath = New ObjectPath("DA://", Nothing)
				AppendWriteableLeafs(SessionToBrowse, FindCriteria, ret, True, NumberToReturn)



			Catch fex As FaultException(Of XiFault)
				If fex.Detail.ErrorCode = XiFaultCodes.E_WRAPPEDSERVER_NOT_ACCESSIBLE Then
					' TODO: Gray out or otherwise handle wrapped server shutdown

				End If
			Catch ex As Exception

			End Try
			Return ret
		End Function


	End Class
End Namespace
